# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################
"""
appels de haut niveau : 
- parsing des logs de dansguardian
- soumissions via xml-rpc
- utilitaires ligne de commande (stdin et raw_input)
"""
import sys

from filtrage.config import *
from filtrage.logparser import parselog
from filtrage.rpc import rpc_add, rpc_submit
from filtrage.retriever import read_url
from filtrage.guesser import Guesser
from filtrage.log import log, logger
from filtrage.errors import URLError, MinimalContentError
from filtrage.tools import url_filter_domain
# XXFIXME 
# from filtrage.db import PickleDB

## filtrage a partir des logs
    
def guess_fromlogs(filename=DANSGUARDIAN_FILE):
    """actions de parsing et d'affectation de pourcentages 
    - parse le fichier de logs de dansguardian
    - récupère chaque page
    - lui affecte un pourcentage
    exemple de sortie :
    (http://url.com, [('porn', 0.94649927972169501), ('noporn', 0.067599721064875107)])
    (http://url.com, [('noporn', 0.99990000000000001)])
    """
    # récuperation des données de log (acces.log)
    datas = parselog(filename, DANSGUARDIAN_FIELDS)
    # récupartion de chaque page
    for data in datas:
        try:
            content = read_url(data['url'])
        except URLError:
            print 'Site non trouvé : %s'%data['url']
            continue
        # affectation d'un pourcentage (test bayesien)
        g = Guesser()
        g.load(BAYES_DATABASE)
        try:
            rate = g.guess_line(content)
        except MinimalContentError:
            print "impossible d'affecter une note a la page %s : contenu trop peu significatif" % data['url']

        url = data['url']
        log(url, rate)
        yield url, rate
      
def rpc_send():
    """utilitaire d'appels xml-rpc qui 
    effectue le filtrage bayesien
    etenvoie des requetes à la base de filtrage
    """
    for url, guesses in guess_fromlogs():
        for keyword, notation in guesses:
            if notation >= BLACKLIST_RATE:
                # liste noire pour l'url automatiquement
                url = url_filter_domain(url)
                try:
                    report = rpc_add(url, keyword, ACAD_NAME, notation)
                    logger.info(url + "retour xml-rpc : " + report)
                except:
                    print "l'appel xml-rpc a echoué"
                    logger.error("l'appel xml-rpc a echoué")

            else:
                if notation >= SUBMIT_RATE:
                    # demande de soumission de l'url
                    url = url_filter_domain(url)
                    try:
                        report = rpc_submit(url, keyword, ACAD_NAME, notation)
                        logger.info(url + "retour xml-rpc : " + report)
                    except:
                        print "l'appel xml-rpc a echoué"
                        logger.error("l'appel xml-rpc a echoué")

## filtrage au prompt
        
def filter_onepage(url):
    """renvoie le pourcentage associé
    à chaque mot clef pour une url donnée
    **url** url de la page à tester
    """
    try:
        content = read_url(url)
        # XXFIXME : faire appel à une moyenne statistique ici
                
        g = Guesser()
        g.load(BAYES_DATABASE)

        # note de filtrage
        rate = 0

        try:
            rate = g.guess_line(content, mode=GUESS_MODE)
        except MinimalContentError:
            print "impossible d'affecter une note a la page %s : contenu trop peu significatif" % url

        log(url, rate)
        return rate

    except Exception, e:
        return e
    
def _filter(url):
    "sortie de la note de filtrage sur la sortie standard" 
    url.strip()
    print url, " : affectation d'une note de filtrage en cours..."
    sys.stdout.write(str(filter_onepage(url))+'\n')
        
def stdin_url_filter():
    "collecte de l'url depuis stdin"
    url = sys.stdin.read()
    _filter(url)
    
def raw_url_filter():
    "collecte de l'url depuis une question au prompt"
    url = raw_input("entrez l'url a noter ou appuyez sur entree (ligne vide) pour quitter: ")
    if url.strip():
        _filter(url)
        raw_url_filter()
    else:
        sys.exit(1)
