# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################
"""
module de manipulation directe sur la base de mots-clefs
"""

import pickle

from filtrage.config import BAYES_DATABASE
from filtrage.errors import DataError

from reverend.thomas import BayesData

# XXFIXME : gestion plus precise de la base bayesienne (pas encore utilise)

class PickleDB:

    def __init__(self, filename=BAYES_DATABASE):
        """filename: nom du fichier de base"""
        self.filename = filename
        self.bayesdict = self._load()

    def add_keyword(self, keyword):
        self.bayesdict[keyword] = BayesData()

    def _load(self):
        """charge la base"""
        fh = file(self.filename, 'r')
        bayesdict = pickle.load(fh)
        return bayesdict

    def save(self):
        fh = file(self.filename, 'w')
        pickle.dump(self.bayesdict, fh)
        fh.close()

    def rate(self):
        pass

    def update_bayesdict(self, other):
        for bayeskey in self.bayesdict.keys():
            if bayeskey != '__Corpus__':
                self.bayesdict[bayeskey].update(other.bayesdict[bayeskey])


def load_pickle(filename=BAYES_DATABASE):
    """charge la base et renvoie un dictionnaire de dictionnaires"""
    fh = file(filename, 'r')
    bayesdict = pickle.load(fh)
    del(bayesdict['__Corpus__'])
    return bayesdict

def add_pickle(keyword, item, filename=BAYES_DATABASE):
    """charge la base et la modifie en ajoutant un mot-clef
    keyword : le mot-clef dans lequel une valeur est à ajouter
    item : (key, value) : le contenu à ajouter à la base
    """
    # ouverture du fichier
    fh = file(filename, 'r')
    bayesdict = pickle.load(fh)
    fh.close()
    if not bayesdict.has_key(keyword): #raise DataError, "le mot-clef doit exister"
        return False
    bayesdict[keyword][item[0]] = item[1]
    # sérialisation dans le fichier
    fh = file(filename, 'w')
    pickle.dump(bayesdict, fh)
    fh.close()
    return True

def del_pickle(keyword, delkey, filename=BAYES_DATABASE):
    """charge la base et la modifie en supprimant un mot-clef
    keyword : le mot-clef dans lequel une valeur est à ajouter
    delkey : le contenu à ajouter à la base
    """
    # ouverture du fichier
    fh = file(filename, 'r')
    bayesdict = pickle.load(fh)
    fh.close()
    if not bayesdict.has_key(keyword): #raise DataError, "le mot-clef doit exister"
        return False
    del(bayesdict[keyword][delkey])
    # sérialisation dans le fichier
    fh = file(filename, 'w')
    pickle.dump(bayesdict, fh)
    fh.close()
    return True
