# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################

"""Module de stockage et de manipulation des fichiers de log de squid 
parsing des logs de dansguardian

tout le fichier est parsé

la lecture, la collecte des données,  (accessoirement le formatage et l'écriture de données) 

adaptation au format des logs : filtrage sur le contenu *EXCEPTION* dans le champ 
"exception" du format de log de dansguardian 

les champs sont définis dans le fichier de configuration

dans le fichier, ne pas soumis les logs qui ont déjà été refusés
par dansguardian (format de log : accepted, rejected)
"""

import os.path , csv

from filtrage.errors import DataError

class CSVStorage:
    """gestion des données issues d'un fichier de type csv
    (exemple : acces.log)"""
    
    def __init__(self, filename=None, keywords=[]):
        """
        filename: chemin du fichier de sauvegarde
        """
        if filename:
            self.filename = filename
            self.storage_file = os.path.abspath(os.path.normpath(self.filename))
        if not keywords:
            raise Exception, "il faut renseigner les noms des champs"
        self.keywords = keywords

    ## chargement et lecture des données
    def load_data(self):
        """Chargement des données à partir du fichier csv
        Le fichier csv est dans le format category,url,id,name,page_url
        """
        error = "erreur au chargement du fichier de log (lecture des urls)"
        # vidons les données
        self.data = []
        # le fichier doit exister
        if not self.storage_file:
            raise DataError, error
        try:
            csv_reader = csv.reader(file(self.storage_file, 'r'))
            for row in csv_reader:
                kwargs = dict(zip(self.keywords, row))
                self.data.append(kwargs)
        except:
            raise DataError, error
        
    def get_data(self):
        """Récupère les données à enregistrer
        """
        return self.data

    ## modification et ecriture des données 
    def write_data(self):
        """Ecriture dans le fichier de stockage
        - vide le fichier
        - ecrit le contenu de self.data par dessus
        """
        fd = file(self.storage_file, 'w')
        writer = csv.writer(fd)
        writer.writerows(self.data)
    
    def filter(self, keyword, exception):
        """filtre les données : si une url est déjà repérée comme interdite
        elle n'est pas gardée dans les données récupérées
        keyword: champ de filtrage
        exception: tag signalant le site interdit dans le champ keyword
        """
        if keyword not in self.keywords:
            raise DataError, "le mot clef de filtrage n'est pas dans la liste des champs"
        
        for data in self.data:
            if exception in data[keyword]:
                self.data.remove(data)
                
    def set_data(self, data):
        """Renseigne les données à enregistrer
        *attention*, remplace **toutes** les données et ecrase
        le self.data existant. Il faut que les données soient
        de même forme que ce qui est renseigné dans self.keywords
        """
        self.data = data


def parselog(filename, keywords):
    """crée un parseur et filtre les résultats
    filename: nom du fichier de log acces.log
    keywords: champs, format du fichier de log
    """
    c = CSVStorage(keywords=keywords, filename=filename) 
    c.load_data()
    """
    **filter_keyword** nom du champ dans lequel se trouve l'exception
    **exception** filtres d'exceptions, phrase clef qui renseigne que dansguardian
    a repéré un site interdit
    """
    filter_keyword = 'exception'
    exception = '*EXCEPTION*'
    c.filter(filter_keyword, exception)
    return c.get_data()

# def _test():
#     # format de log du fichier à parcourir
#     keywords = ['date','login','ip', 'url', 'method', 'chiffre1', 'chiffre2']
#     c = CSVStorage(keywords=keywords, filename="essai.txt")
#     data = [["2007.4.12 17:44:39","toto.titi","172.16.88.20","http://www.google.com","","GET","218","0","","1","302","text/html","",""]]
#     c.set_data(data)
#     c.write_data()

# if __name__ == "__main__":
#     _test()

